#!/usr/bin/python3

import argparse
import logging

from colorama import Fore, Style

from nlab_inf_engine_scripts.InfEngineServer import InfEngineServer
from nlab_inf_engine_scripts.version import VERSION


def main():
    def success():
        print(f"    {Fore.GREEN}[SUCCESS]{Style.RESET_ALL}\n")

    def failure():
        print(f"    {Fore.RED}[FAILED]{Style.RESET_ALL}\n")

    parser = argparse.ArgumentParser(description="Manipulate InfEngine server.")
    parser.add_argument(
        "--root-path",
        action="store",
        nargs=1,
        default=None,
        metavar="ROOT_PATH",
        help=f"Default path: {InfEngineServer().root_path}",
    )

    group = parser.add_argument_group("Logging").add_mutually_exclusive_group()
    group.add_argument(
        "-v",
        "--verbose",
        action="store_true",
        help=f"Enable INFO level logging.",
    )
    group.add_argument(
        "-vv",
        action="store_true",
        help=f"Enable DEBUG level logging.",
    )

    group = parser.add_argument_group("DL Compilation").add_mutually_exclusive_group()
    group.add_argument(
        "--dl-update",
        action="store",
        default=None,
        metavar="DL_DATA_PATH",
        help=f"Update DL data (binary or sources).",
    )
    group.add_argument(
        "--dl-delete",
        action="store",
        default=None,
        metavar="DL_DATABASE_PATH",
        help=f"Delete given DL data.",
    )
    group.add_argument(
        "--dl-delete-all", action="store_true", help=f"Delete all DL data."
    )

    group = parser.add_argument_group("DL Compilation Options")
    group.add_argument(
        "--db-name",
        action="store",
        default="dldata",
        help=f"Specify DL binary name. Default: dldata",
    )
    group.add_argument(
        "--dl-verbose",
        action="store_true",
        help=f"Enable DEBUG level logging.",
    )

    group = parser.add_argument_group("InfServer").add_mutually_exclusive_group()
    group.add_argument("--start", action="store_true", help=f"Start InfServer.")
    group.add_argument("--stop", action="store_true", help=f"Stop InfServer.")
    group.add_argument("--restart", action="store_true", help=f"Restart InfServer.")
    group.add_argument("--respawn", action="store_true", help=f"Respawn InfServer.")
    group.add_argument("--status", action="store_true", help=f"Get InfServer status.")
    group.add_argument(
        "--version", action="store_true", help=f"Print package version and exit."
    )
    group.add_argument(
        "--engine-version", action="store_true", help=f"Print engine version and exit."
    )

    args = parser.parse_args()

    root_path = args.root_path[0] if args.root_path else None

    enable_logging = False
    if args.verbose:
        logging_level = logging.INFO
        enable_logging = True
    elif args.vv:
        logging_level = logging.DEBUG
        enable_logging = True
    else:
        logging_level = logging.ERROR

    if not args.version and not args.engine_version:
        print("\n    InfEngine Manager\n")

    server = InfEngineServer(
        root_path=root_path, logging_level=logging_level, enable_logging=enable_logging
    )

    try:
        if args.start:
            print(" * Starting InfServer", end="")
            server.start()
            success()
        elif args.stop:
            print(" * Stopping InfServer", end="")
            server.stop()
            success()
        elif args.restart:
            print(" * Restarting InfServer", end="")
            server.restart()
            success()
        elif args.respawn:
            print(" * Respawning InfServer", end="")
            server.respawn()
            success()
        elif args.status:
            print(" * Getting InfServer status", end="")
            status = server.status()
            success()
            print(f"InfServer status: {status}\n")
        elif args.version:

            print(VERSION)
        elif args.engine_version:

            print(f"{server.version()}")
        elif args.dl_update:
            print(" * Update data", end="")
            server.update_data(dldata=args.dl_update, db_name=args.db_name)
            success()
        elif args.dl_delete:
            print(" * Delete data", end="")
            server.delete_dl(database_name=args.dl_delete)
            success()
        elif args.dl_delete_all:
            print(" * Delete all data", end="")
            server.delete_dl_all()
            success()
        else:
            parser.print_help()
            print()
    except Exception as e:
        failure()
        print(f"\n{e}\n")


if __name__ == "__main__":
    main()



import datetime
import random
import string
import subprocess
import tempfile
from pathlib import Path


def create_temp_dir(parent_dir: Path) -> Path:  # tempfile.TemporaryDirectory:
    """ Create a temp directory, which will be deleted on object destruction. """

    current_time = datetime.datetime.now()

    # temp_dir = tempfile.TemporaryDirectory(
    #     prefix=f"{current_time.year}-{current_time.month}-{current_time.day}_{current_time.hour}-{current_time.minute}_",
    #     dir=parent_dir,
    # )

    randomized_name = "".join(random.choices(string.ascii_letters, k=6))
    temp_dir = (
        parent_dir / f"{datetime.datetime.now().strftime('%Y-%m-%d_%H-%M')}"
        f"_{randomized_name}"
    )
    temp_dir.mkdir(parents=True)

    return temp_dir


def symlink(frm: Path, to: Path, force: bool = False):
    """ Create a symlink from one file to another. """

    if force:
        cmd = f"ln -sfT {frm} {to}"
    else:
        cmd = f"ln -s {frm} {to}"

    completed_process = subprocess.run(cmd, shell=True)

    if completed_process.returncode:
        raise Exception(f"Can't create a symlink: {cmd}")

    return to



import logging
import shutil
import subprocess
from configparser import ConfigParser
from pathlib import Path
from time import sleep
from typing import Optional, Union

import memcache

# Paths.
from nlab_inf_engine_scripts.functions import create_temp_dir
from nlab_inf_engine_scripts.InfEngineServerEngine import InfEngineServerEngine

INF_ENGINE_PATH_DATA = "data/"
INF_ENGINE_PATH_TMP = "tmp/"
INF_ENGINE_PATH_CONF = "conf/"
INF_ENGINE_PATH_BIN = "bin/"
INF_ENGINE_PATH_LOGS = "logs/"

INF_ENGINE_PATH_ENGINES = INF_ENGINE_PATH_DATA + "engine"
INF_ENGINE_PATH_RELEASES = INF_ENGINE_PATH_DATA + "release"

# Links.
INF_ENGINE_LINK_READY = "ready"
INF_ENGINE_LINK_RELEASE = "release"

# Filenames.
INF_ENGINE_DLDATA_FILE = "dldata.ie2"
INF_ENGINE_ELDATA_FILE = "eldata.ie2"

# Default root InfEngine server directory.
INF_ENGINE_DEFAULT_ROOT_DIRECTORY = "/usr/local/InfEngine"

# Config paths.
INF_ENGINE_SERVER_CONFIG = INF_ENGINE_PATH_CONF + "InfServer.conf"
AP_PROCESS_SERVER_CONFIG = INF_ENGINE_PATH_CONF + "ap-process-server.conf"
INF_ENGINE_COMPILER_CONFIG = INF_ENGINE_PATH_CONF + "InfCompiler.conf"
INF_ENGINE_ENVIRONMENT_CONFIG = INF_ENGINE_PATH_CONF + "Environment.conf"
INF_ENGINE_ELLIPSIS_COMPILER_CONFIG = INF_ENGINE_PATH_CONF + "EllipsisCompiler.conf"

# Binaries paths.
AP_PROCESS_SERVER_BINARY = INF_ENGINE_PATH_BIN + "ap-process-server"

# Lockfile paths.
INF_ENGINE_LOCK_FILE = INF_ENGINE_PATH_TMP + "BuildEngine.lock"

# Log file.
INF_ENGINE_LOG_FILE_PATH = INF_ENGINE_PATH_LOGS + "InfEngine.log"

# Default config values.
DEFAULT_ENVIRONMENT_CLEAN_RELEASES = 1
DEFAULT_ENVIRONMENT_CLEAN_ENGINES = 1

# Default memcached server.
DEFAULT_INF_SERVER_CACHE_SERVERS = "localhost:11211"


class InfEngineServer:
    root_path: Path
    _config: dict
    _config_path: dict
    _logger: logging.Logger

    def __init__(
        self,
        root_path: Path = None,
        logging_level: int = logging.ERROR,
        enable_logging: bool = False,
    ):
        """ Class that handles InfEngineServer commands. """

        # Determine the root path
        if root_path:
            self.root_path = Path(root_path)
        else:
            self.root_path = Path(__file__).parent.parent

            if not (
                Path(self.root_path / INF_ENGINE_SERVER_CONFIG).is_file()
                or Path(self.root_path / AP_PROCESS_SERVER_CONFIG).is_file()
                or Path(self.root_path / INF_ENGINE_COMPILER_CONFIG).is_file()
                or Path(self.root_path / INF_ENGINE_ENVIRONMENT_CONFIG).is_file()
            ):
                self.root_path = Path(INF_ENGINE_DEFAULT_ROOT_DIRECTORY)

        # Create directories
        Path(self.root_path / INF_ENGINE_PATH_TMP).mkdir(exist_ok=True)
        Path(self.root_path / INF_ENGINE_PATH_LOGS).mkdir(exist_ok=True)
        Path(self.root_path / "release" / "dldata").mkdir(exist_ok=True)

        # Create a logger
        self._logger = logging.getLogger("InfEngineServer")
        self._logger.handlers = []

        formatter = logging.Formatter(
            fmt="%(asctime)-30s %(levelname)-10s %(message)s",
            datefmt="%Y.%m.%d %H:%M:%S",
        )

        log_file_name = Path(self.root_path / INF_ENGINE_LOG_FILE_PATH)
        log_file_name.touch(exist_ok=True)

        handler = logging.FileHandler(
            filename=Path(self.root_path / INF_ENGINE_LOG_FILE_PATH)
        )
        handler.setFormatter(formatter)
        self._logger.addHandler(handler)

        if enable_logging:
            formatter = logging.Formatter(fmt=" * %(message)s")
            handler = logging.StreamHandler()
            handler.setFormatter(formatter)
            self._logger.addHandler(handler)

        self._logger.setLevel(logging_level)

        # Config
        self._config = {}
        self._config_path = {}

    def _load_config(
        self, config_type: str, force: bool = False
    ) -> Optional[Union[ConfigParser, dict]]:
        """
        Load InfEngine config.

        :param config_type: Config type. Can be:
         'inf_server',
         'inf_compiler',
         'process_server',
         'environment'.
        :param force: Force config re-load if it exists.
        :return: Config dict.
        """

        # Check if config is already loaded
        if not force and config_type in self._config:
            return self._config[config_type]

        self._logger.debug(f"Loading config: {config_type}.")

        # Get config path
        config_path = None

        if config_type == "inf_server":
            config_path = self.root_path / INF_ENGINE_SERVER_CONFIG
        elif config_type == "inf_compiler":
            config_path = self.root_path / INF_ENGINE_COMPILER_CONFIG
        elif config_type == "process_server":
            config_path = self.root_path / AP_PROCESS_SERVER_CONFIG
        elif config_type == "environment":
            config_path = self.root_path / INF_ENGINE_ENVIRONMENT_CONFIG

        self._logger.debug(f"Config path: {config_path}.")

        # Read config
        # TODO: Formatting dependant config parsing. Should change later.
        config = {}
        if config_type in ["inf_server", "inf_compiler", "environment"]:
            with Path(config_path).open("r") as f:
                lines = f.read().replace("// ", "#")

            config = ConfigParser()
            config.read_string(lines)

            config = {s: dict(config.items(s)) for s in config.sections()}
        elif config_type == "process_server":
            with Path(config_path).open("r") as f:
                for line in f.readlines():
                    line = line.strip()

                    if not line or line.startswith("#"):
                        continue
                    else:
                        key, value = line.split(" ", 1)

                    config[key] = value

        self._logger.debug(f"Config: {config}")

        # Modify values
        if config_type == "inf_server":
            config["Cache"]["servers"] = (
                config["Cache"]["servers"] or DEFAULT_INF_SERVER_CACHE_SERVERS
            ).split(",")
        elif config_type == "inf_compiler":
            pass
        elif config_type == "process_server":
            config["PidFile"] = Path(self.root_path / str(config["PidFile"]))
        elif config_type == "environment":
            if "Clean" in config:
                if "releases" not in config["Clean"]:
                    config["Clean"]["releases"] = DEFAULT_ENVIRONMENT_CLEAN_RELEASES

                if "engines" not in config["Clean"]:
                    config["Clean"]["engines"] = DEFAULT_ENVIRONMENT_CLEAN_ENGINES
            else:
                config["Clean"] = {
                    "releases": DEFAULT_ENVIRONMENT_CLEAN_RELEASES,
                    "engines": DEFAULT_ENVIRONMENT_CLEAN_ENGINES,
                }

        self._logger.debug(f"Config: {config}")
        self._config[config_type] = config
        self._config_path[config_type] = config_path

        return config

    def _check(self, pid: int) -> Optional[int]:
        """ Get InfEngine server process info. """

        completed_process = subprocess.run(
            f"ps -p {pid} --no-headers", shell=True, stdout=subprocess.PIPE
        )

        self._logger.debug(f"Process: {completed_process.stdout}")

        if completed_process.stdout:
            return pid
        else:
            return None

    def _pid(self) -> Optional[int]:
        """ Get InfEngine server pid. """

        config = self._load_config("process_server")

        pidfile = Path(config["PidFile"])

        self._logger.debug(f"Pid file path: {pidfile}")

        if not pidfile.is_file():
            return None

        self._logger.debug(f"Pid file exists.")

        try:
            with pidfile.open("r") as f:
                pid = f.read().strip()
        except Exception as e:
            self._logger.error(f"Can't open pidfile '{pidfile}': {e}")

            raise e

        if not pid:
            return None
        else:
            try:
                pid = int(pid)
            except Exception as e:
                self._logger.error(
                    f"Invalid InfEngine server pid file '{pidfile}': {pid}"
                )

        self._logger.debug(f"Pid: {pid}.")

        return self._check(pid)

    # def _clean(self, dir: Path, exclude: list = None, limit: int = None):
    #     counter = 0
    #
    #     if not dir.is_dir():
    #         return counter
    #
    #     files = []
    #     for file in dir.iterdir():
    #         pass

    # def clean(self):
    #     """ Clean old releases, DL/ellipsis data and engines. """
    #
    #     self._logger.info("Cleaning InfServers.")
    #
    #     config = self._load_config("environment")
    #
    #     # Clean engines
    #     """
    #     # Clean engines.
    #     my $count = _clean(
    #             $self->{root_path} + INF_ENGINE_PATH_ENGINES,
    #             $self->{root_path} + INF_ENGINE_LINK_READY,
    #             $config->get( 'Clean', 'Engines' ) );
    #     if( $count ) {
    #         INFO "Deleted $count engines.";
    #     };
    #     """

    def start(self):
        """ Start InfEngine server. """

        self._logger.info("Starting InfEngine server.")

        if self._pid():
            self._logger.warning("Can't start InfEngine server: it is already running.")
            return

        self._load_config("process_server")

        res = f"cd '{self.root_path}' && {self.root_path / AP_PROCESS_SERVER_BINARY} {self._config_path['process_server']} 2>&1"

        config = self._load_config("inf_server")

        self._logger.debug("Flushing Memcached servers.")
        memcache.Client(servers=config["Cache"]["servers"]).flush_all()

        try:
            self._logger.debug(f"Trying to start the server: {res}")
            completed_process = subprocess.run(res, shell=True)
        except subprocess.TimeoutExpired as e:
            self._logger.error("Can't start InfEngine server: timeout is expired.")
            raise Exception(f"Can't start InfEngine server: timeout is expired.")
        else:
            if completed_process.returncode != 0:
                self._logger.error(
                    f"Can't start InfEngine server: {res}, {completed_process.returncode}"
                )
                raise Exception(
                    f"Can't start InfEngine server: {res}, {completed_process.returncode}"
                )

        # Sleep until the server is started
        while True:
            if not self._pid():
                sleep(1)
            else:
                break

        self._logger.info("InfEngine server has started.")

    def stop(self):
        """ Stop InfEngine server. """

        self._logger.info("Stopping InfEngine server.")

        pid = self._pid()
        if not pid:
            self._logger.warning("Can't stop InfEngine server: it isn't running.")
            return

        # Try to kill the process
        res = f"kill -TERM {pid}"

        try:
            self._logger.debug(f"Trying to kill the server: {res}")
            completed_process = subprocess.run(res, shell=True)
        except subprocess.TimeoutExpired as e:
            self._logger.error("Can't kill InfEngine server: timeout is expired.")
            return
        else:
            if completed_process.returncode != 0:
                self._logger.error(f"Can't stop InfEngine server: {res}")
                return

        # Sleep until the server is stopped
        while True:
            if self._check(pid):
                sleep(1)
            else:
                break

        self._logger.info("InfEngine server has stopped.")

    def restart(self):
        """ Restart InfEngine server. """

        self._logger.info("Restarting InfEngine server.")

        if self._pid():
            self.stop()

            sleep(5)

            self.start()
        else:
            self._logger.warning("Can't restart InfEngine server: it isn't running.")

    def respawn(self):
        """ Respawn InfEngine workers. """

        self._logger.info("Respawning InfEngine workers.")

        pid = self._pid()
        if not pid:
            self._logger.warning(
                "Can't respawn InfEngine workers: server wasn't started."
            )
            return

        res = f"kill -HUP {pid}"

        try:
            self._logger.debug(f"Trying to kill the server: {res}")
            completed_process = subprocess.run(res, shell=True)
        except subprocess.TimeoutExpired as e:
            self._logger.error("Can't kill InfEngine server: timeout is expired.")
            return
        else:
            if completed_process.returncode != 0:
                self._logger.error(f"Can't stop InfEngine server: {res}")
                return

        sleep(10)

        self._logger.info("InfEngine workers have respawned.")

    def status(self):
        """ Check InfEngine server status. """

        if self._pid():
            return "RUNNING"
        else:
            return "NOT RUNNING"

    def clean(self):
        self._logger.info(f"Cleaning InfServers...")

        config = self._load_config("environment")

        count = self._clean(
            self.root_path / INF_ENGINE_PATH_ENGINES,
            self.root_path / INF_ENGINE_LINK_READY,
            int(config["Clean"]["releases"]),
        )

        if count:
            self._logger.info(f"Deleted {count} engines.")

        count = self._clean(
            self.root_path / INF_ENGINE_PATH_RELEASES,
            self.root_path / INF_ENGINE_LINK_RELEASE,
            int(config["Clean"]["releases"]),
        )

        if count:
            self._logger.info(f"Deleted {count} releases.")

    def _clean(self, dir: Path, ignore: Path, limit: int = 0) -> int:
        # TODO
        pass

    def _clean_multidb(self):
        # TODO
        pass

    def update_data(
        self,
        dldata: str = None,
        dlsources: str = None,
        strict: bool = False,
        db_name: str = "dldata",
    ):
        """ Update DL data. """

        self._logger.info("Updating Data.")

        if Path(dldata).is_file():
            dldata = Path(dldata)

            self._logger.info(f"DL data path: {dldata}")
        elif Path(dldata).is_dir():
            dlsources = Path(dldata)
            dldata = None

            self._logger.info(f"DL sources path: {dlsources}")
        else:
            self._logger.error(f"Invalid path: {dldata}")
            raise Exception(f"Invalid path: {dldata}")

        # TODO: lock

        engine = InfEngineServerEngine(
            self.root_path / INF_ENGINE_LINK_RELEASE, logger=self._logger
        )

        # temp_dir = create_temp_dir(self.root_path / INF_ENGINE_PATH_TMP)
        # temp_dir_path = Path(temp_dir.name)
        temp_dir_path = create_temp_dir(self.root_path / INF_ENGINE_PATH_TMP)
        self._logger.debug(f"Creating a temp dir: {temp_dir_path}.")

        if dldata:
            if engine.check(dldata) == 0:
                self._logger.info(f"Updating DL data binary: {dldata}")
            else:
                self._logger.error(f"DL data is incompatible.")
                raise Exception(f"DL data is incompatible.")
        elif dlsources:
            self._logger.info(f"Updating DL data from sources: {dlsources}")

            dldata = temp_dir_path / INF_ENGINE_DLDATA_FILE
            engine.compile_dldata(
                dlsources, self.root_path / INF_ENGINE_COMPILER_CONFIG, dldata, strict
            )

        engine.install(dldata, dlsources, db_name=db_name)

        self.restart()

        # TODO: clean and unlock

    def delete_dl(self, database_name):
        """ Delete DL databases. """

        file = (
            self.root_path / INF_ENGINE_LINK_RELEASE / "dldata" / f"{database_name}.ie2"
        )

        self._logger.info(f"Deleting database: {file}")

        file.unlink()

    def delete_dl_all(self):
        """ Delete DL databases. """

        directory = self.root_path / INF_ENGINE_LINK_RELEASE / "dldata"

        self._logger.info(f"Deleting all databases from: {directory}")

        for obj in directory.iterdir():
            self._logger.debug(f"Deleting: {obj}")
            if obj.is_file():
                obj.unlink()
            elif obj.is_dir():
                shutil.rmtree(obj)

    def version(self):
        """ Get InfServer version. """

        engine = InfEngineServerEngine(
            self.root_path / INF_ENGINE_LINK_RELEASE, logger=self._logger
        )

        return engine.version()



import logging
import re
import shutil
import subprocess
from pathlib import Path

# Paths.
from nlab_inf_engine_scripts.functions import create_temp_dir, symlink

INF_ENGINE_PATH_BIN = "bin/"
INF_ENGINE_PATH_CONF = "conf/"
INF_ENGINE_PATH_LIB = "lib/"
INF_ENGINE_PATH_BUILD = "build/"
INF_ENGINE_PATH_DLDATA = "dldata/"
INF_ENGINE_PATH_DLSOURCES = "sources/"
INF_ENGINE_PATH_ELDATA = "eldata/"
INF_ENGINE_PATH_ELSOURCES = "sources/"

INF_ENGINE_PATH_FUNCTIONS = INF_ENGINE_PATH_LIB + "functions/"

INF_ENGINE_PATH_BUILD_BIN = INF_ENGINE_PATH_BUILD + "bin/"
INF_ENGINE_PATH_BUILD_LIB = INF_ENGINE_PATH_BUILD + "lib/"

# Binaries.
INF_ENGINE_BINARY_COMPILER = INF_ENGINE_PATH_BIN + "InfCompiler"
INF_ENGINE_BINARY_SERVER = INF_ENGINE_PATH_BIN + "InfServer"
INF_ENGINE_BINARY_ELLIPSIS_CMP = INF_ENGINE_PATH_BIN + "EllipsisCompiler"
INF_ENGINE_BINARY_CHECK_SIGNATURE = INF_ENGINE_PATH_BIN + "CheckSignature"
AP_PROCESS_SERVER_BINARY = INF_ENGINE_PATH_BIN + "ap-process-server"

# Binaries sets.
INF_ENGINE_BINARIES_SERVER = [
    AP_PROCESS_SERVER_BINARY,
    INF_ENGINE_PATH_BIN + "InfServer",
    INF_ENGINE_PATH_BIN + "CheckSignature",
]

INF_ENGINE_BINARIES_COMPILERS = [
    INF_ENGINE_PATH_BIN + "InfCompiler",
    INF_ENGINE_PATH_BIN + "EllipsisCompiler",
]
INF_ENGINE_BINARIES_TOOLS = [
    INF_ENGINE_PATH_BIN + "InfServer",
]

# Libraries sets.
INF_ENGINE_LIBRARIES = [
    INF_ENGINE_PATH_LIB + "libClientLib.a",
    INF_ENGINE_PATH_LIB + "libClientLib.dylib",
    INF_ENGINE_PATH_LIB + "libClientLib.so",
]

# Config paths.
INF_ENGINE_CONFIG_FUNCTIONS = INF_ENGINE_PATH_CONF + "functions.lst"

# Files.
INF_ENGINE_FILE_BUILD = "HEAD/InfEngine2/Build.hpp"
INF_ENGINE_FILE_DLDATA = "dldata.ie2"
INF_ENGINE_FILE_ELDATA = "eldata.ie2"
INF_ENGINE_FILE_ALIASES = "aliases.dl"
INF_ENGINE_FILE_ENCODING = "encoding.info"

# Links.
INF_ENGINE_LINK_DLDATA = INF_ENGINE_PATH_DLDATA + "dldata.ie2"
INF_ENGINE_LINK_DLSOURCES = INF_ENGINE_PATH_DLDATA + "dlsources"
INF_ENGINE_LINK_ELDATA = INF_ENGINE_PATH_ELDATA + "eldata.ie2"
INF_ENGINE_LINK_ELSOURCES = INF_ENGINE_PATH_ELDATA + "elsources"
INF_ENGINE_LINK_ALIASES = INF_ENGINE_PATH_DLDATA + "aliases.dl"


class InfEngineServerEngine:
    _engine_dir: Path
    _logger: logging.Logger

    def __init__(self, engine_dir: Path, logger: logging.Logger):
        self._engine_dir = engine_dir

        self._logger = logger

    def check(self, dldata: Path = None):
        """ Check compatibility target DL and/or ellipsis data and InfEngine. """

        if not Path(self._engine_dir / INF_ENGINE_BINARY_CHECK_SIGNATURE).is_file():
            return 0

        cmd = (
            f"{self._engine_dir / INF_ENGINE_BINARY_CHECK_SIGNATURE}"
            f" --inf-server={self._engine_dir / INF_ENGINE_BINARY_SERVER}"
            f" --functions={self._engine_dir / INF_ENGINE_CONFIG_FUNCTIONS}"
            f" --functions-root={self._engine_dir / INF_ENGINE_PATH_FUNCTIONS}"
            f" --base={dldata}"
            f" 2>&1"
        )

        completed_process = subprocess.run(cmd, shell=True)

        return completed_process.returncode

    def compile_dldata(
        self, dlsources: str, configfile: Path, targetfile: Path, strict: bool
    ):
        """ Compile DL date from sources. """

        self._logger.info(f"Compiling DL data: {dlsources}")

        if not Path(self._engine_dir / INF_ENGINE_BINARY_COMPILER).is_file():
            self._logger.error(
                f"Invalid InfCompiler path: {self._engine_dir / INF_ENGINE_BINARY_COMPILER}"
            )
            raise Exception("Invalid InfCompiler path.")

        aliases_file = Path(dlsources) / Path(INF_ENGINE_LINK_ALIASES)
        if not aliases_file.is_file():
            aliases_file = self._engine_dir / Path(INF_ENGINE_LINK_ALIASES)

            if not aliases_file.is_file():
                aliases_file = None

        encoding = ""
        encoding_file = Path(Path(dlsources) / INF_ENGINE_FILE_ENCODING)
        if encoding_file.is_file():
            with encoding_file.open("r") as f:
                source = f.read()

            if re.fullmatch("^UTF(-?)8$", source):
                encoding = "utf-8"
            elif re.fullmatch("^(CP|WIN)(-?)1251$", source):
                encoding = "cp1251"

        cmd = (
            f"{self._engine_dir / INF_ENGINE_BINARY_COMPILER}"
            f" --dldata-root={dlsources}"
            f" --functions-root={self._engine_dir / INF_ENGINE_PATH_FUNCTIONS}"
            f" --target-path={targetfile}"
            f" --strict={'true' if strict else 'false'}"
        )

        if aliases_file:
            cmd += f" --dlaliases {aliases_file}"
        if encoding:
            cmd += f" --encoding {encoding}"

        cmd += f" {configfile} 2>&1"

        completed_process = subprocess.run(cmd, shell=True)

        if completed_process.returncode:
            targetfile.unlink()

            self._logger.error(f"Can't compile DL data: {cmd}")
            raise Exception("Can't compile DL data.")

        self._logger.info(f"DL data compiled.")

    def install(
        self,
        dldata: Path,
        dlsources: Path = None,
        aliases: Path = None,
        db_name: str = "dldata",
    ):
        """ Install DL data to InfEngine. """

        self._logger.info(f"Installing database: {db_name}")

        INF_ENGINE_FILE_DLDATA_VAR = f"{db_name}.ie2"
        INF_ENGINE_LINK_DLDATA_VAR = INF_ENGINE_PATH_DLDATA + INF_ENGINE_FILE_DLDATA_VAR

        # dldata_obj = create_temp_dir(self._engine_dir / INF_ENGINE_PATH_DLDATA)
        # dldata_path = Path(dldata_obj.name)
        dldata_path = create_temp_dir(self._engine_dir / INF_ENGINE_PATH_DLDATA)

        self._logger.debug(f"dldata_path: {dldata_path}")

        # Install aliases
        if not aliases:
            self._logger.debug(
                f"Aliases were not passed."
                f" Copying {self._engine_dir / INF_ENGINE_LINK_ALIASES}"
                f" to {dldata_path / INF_ENGINE_FILE_ALIASES}"
            )
            if Path(self._engine_dir / INF_ENGINE_LINK_ALIASES).is_file():
                shutil.copy(
                    self._engine_dir / INF_ENGINE_LINK_ALIASES,
                    dldata_path / INF_ENGINE_FILE_ALIASES,
                )
        elif aliases.is_file():
            self._logger.debug(
                f"Aliases were passed."
                f" Copying {aliases}"
                f" to {dldata_path / INF_ENGINE_FILE_ALIASES}"
            )
            shutil.copy(aliases, dldata_path / INF_ENGINE_FILE_ALIASES)
        else:
            raise Exception(f"Invalid aliases path: {aliases}")

        # Install DL data
        if not dldata.is_file():
            raise Exception(f"Invalid DL data path: {dldata}")
        self._logger.debug(
            f"DL data is a file."
            f" Copying {dldata}"
            f" to {dldata_path / INF_ENGINE_FILE_DLDATA_VAR}"
        )
        shutil.copy(dldata, dldata_path / INF_ENGINE_FILE_DLDATA_VAR)

        # Install DL data sources
        if dlsources:
            if not dlsources.is_dir():
                raise Exception(f"Invalid DL data sources path: {dlsources}")
            self._logger.debug(
                f"DL sources is a directory."
                f" Copying {dlsources}"
                f" to {dldata_path / INF_ENGINE_PATH_DLSOURCES}"
            )
            shutil.copytree(dlsources, dldata_path / INF_ENGINE_PATH_DLSOURCES)

            # Install aliases from DL data sources
            sources_aliases = dlsources / INF_ENGINE_FILE_ALIASES

            if not sources_aliases.is_file():
                raise Exception(
                    f"Invalid aliases in DL data sources: {sources_aliases}"
                )

            if not aliases:
                self._logger.debug(
                    f"Copying {sources_aliases}"
                    f" to {dldata_path / INF_ENGINE_FILE_ALIASES}"
                )
                shutil.copy(sources_aliases, dldata_path / INF_ENGINE_FILE_ALIASES)

        # Switch softlinks
        symlink_from = (
            self._engine_dir
            / INF_ENGINE_PATH_DLDATA
            / dldata_path
            / INF_ENGINE_FILE_DLDATA_VAR
        )
        self._logger.debug(
            f"Creating a symlink from {symlink_from}"
            f" to {self._engine_dir / INF_ENGINE_LINK_DLDATA}"
        )
        symlink(
            symlink_from,
            self._engine_dir / (INF_ENGINE_PATH_DLDATA + INF_ENGINE_FILE_DLDATA_VAR),
            force=True,
        )

        if db_name == "dldata":
            if Path(dldata_path / INF_ENGINE_PATH_DLSOURCES).is_file():
                symlink_from = (
                    self._engine_dir
                    / INF_ENGINE_PATH_DLDATA
                    / dldata_path
                    / INF_ENGINE_PATH_DLSOURCES
                )
                self._logger.debug(
                    f"Creating a symlink from {symlink_from}"
                    f" to {self._engine_dir / INF_ENGINE_LINK_DLSOURCES}"
                )
                symlink(
                    self._engine_dir
                    / INF_ENGINE_PATH_DLDATA
                    / dldata_path
                    / INF_ENGINE_PATH_DLSOURCES,
                    self._engine_dir / INF_ENGINE_LINK_DLSOURCES,
                    force=True,
                )
            else:
                self._logger.debug(
                    f"Removing file {self._engine_dir / INF_ENGINE_LINK_DLSOURCES}"
                )
                if Path(self._engine_dir / INF_ENGINE_LINK_DLSOURCES).is_file():
                    Path(self._engine_dir / INF_ENGINE_LINK_DLSOURCES).unlink()

            if Path(dldata_path / INF_ENGINE_FILE_ALIASES).is_file():
                symlink_from = (
                    self._engine_dir
                    / INF_ENGINE_PATH_DLDATA
                    / dldata_path
                    / INF_ENGINE_FILE_ALIASES
                )
                self._logger.debug(
                    f"Creating a symlink from {symlink_from}"
                    f" to {self._engine_dir / INF_ENGINE_LINK_ALIASES}"
                )
                symlink(
                    symlink_from,
                    self._engine_dir / INF_ENGINE_LINK_ALIASES,
                    force=True,
                )
            else:
                self._logger.debug(
                    f"Removing file {self._engine_dir / INF_ENGINE_LINK_ALIASES}"
                )
                if Path(self._engine_dir / INF_ENGINE_LINK_DLSOURCES).is_file():
                    Path(self._engine_dir / INF_ENGINE_LINK_DLSOURCES).unlink()

    def version(self):
        """ Get InfServer version. """

        server = self._engine_dir / INF_ENGINE_BINARY_SERVER

        self._logger.debug(f"Trying to check the server: {server}")

        if server.is_file():
            res = f"{server} --version"

            completed_process = subprocess.run(res, shell=True, stdout=subprocess.PIPE)
            process_stdout = completed_process.stdout.decode("utf-8")

            if completed_process.returncode:
                self._logger.error(f"Can't get InfEngine version: {res}")
                raise Exception(f"Can't get InfEngine version: {res}")
            elif process_stdout.find("Release version:") != -1:
                full_version = re.search(
                    "Release version: ([0-9a-zA-Z.-]+)", process_stdout
                )

                if full_version:
                    version = full_version.group().replace("Release version: ", "")

                    return version

        return None



#!/usr/bin/env python

from distutils.core import setup

from nlab_inf_engine_scripts.version import VERSION


setup(
    name='nlab_inf_engine_scripts',
    packages=['nlab_inf_engine_scripts'],
    version=VERSION,
    license='MIT',
    description='Python scripts for manipulation InfEngine.',
    author='Dmitry Makarov',
    author_email='dmakarov@nanosemantics.ru',
    install_requires=[
        'python-memcached',
        'colorama'
    ],
    entry_points={
        'console_scripts': [
            "InfEngineControl = nlab_inf_engine_scripts.InfEngineControl:main"
        ]
    }
)
